import sys
import cherrypy

import json
json_decode = json.JSONDecoder().decode
json_encode = json.JSONEncoder().iterencode


def json_in(force=True, debug=False):
    request = cherrypy.serving.request
    def json_processor(entity):
        """Read application/json data into request.json."""
        if not entity.headers.get("Content-Length", ""):
            raise cherrypy.HTTPError(411)
        
        body = entity.fp.read()
        try:
            request.json = json_decode(body.decode('utf-8'))
        except ValueError:
            raise cherrypy.HTTPError(400, 'Invalid JSON document')
    if force:
        request.body.processors.clear()
        request.body.default_proc = cherrypy.HTTPError(
            415, 'Expected an application/json content type')
    request.body.processors['application/json'] = json_processor

def json_out(debug=False):
    request = cherrypy.serving.request
    response = cherrypy.serving.response
    
    real_handler = request.handler
    def json_handler(*args, **kwargs):
        response.headers['Content-Type'] = 'application/json'
        value = real_handler(*args, **kwargs)
        def encoder():
            for chunk in json_encode(value):
                yield chunk.encode('utf8')
        return encoder()
    request.handler = json_handler
